#include "piano.hpp"

PianoKeyboard::PianoKeyboard(int x, int y, int octave) :
	Icon(x, y,
	 x + WHITE_KEY_WIDTH * WHITE_KEYS_PER_OCTAVE,
	 y + WHITE_KEY_HEIGHT)
{
	// create the white keys first
	CreateWhiteKeys(x, y, octave);

	// then overlay the black keys between them
	CreateBlackKeys(x, y, octave);
}

void PianoKeyboard::CreateWhiteKeys(int x, int y, int octave)
{
	// white notes in the middle DO octave:
	// DO, RE, MI, FA, SOL, LA, TI
	static int white_notes [] = {
	  532, 587, 659, 698, 784, 880, 988
	};
	int key_x;

	for (int note = 0;
	     note < WHITE_KEYS_PER_OCTAVE;
	     note++) {
	  key_x = x + note * (WHITE_KEY_WIDTH + 1);
	  key [note] = new PianoKey(this, key_x, y,
				 WHITE_KEY_WIDTH,
				 WHITE_KEY_HEIGHT,
				 WHITE, octave * white_notes [note] );
	}
}

void PianoKeyboard::CreateBlackKeys(int x, int y, int octave)
{
	int key_x, key_y;
	// black notes in the middle DO octave:
	// DO#, RE#, (leave room for MI#), FA#, SOL#, LA#
	static int black_notes [] = {
	  554, 622, 698, 740, 831, 932
	};
	int i, note;

	// overlay the black keys bewteen the white ones
	for (i = 0, note = WHITE_KEYS_PER_OCTAVE;
	     note < KEYS_PER_OCTAVE;
	     i++, note++) {

	  // there's no black between MI and FA
	  if (note == 9) continue;

	  key_x = x + (note - WHITE_KEYS_PER_OCTAVE) *
		  (WHITE_KEY_WIDTH + 1) +
		  WHITE_KEY_WIDTH - BLACK_KEY_WIDTH / 2;
	  key_y = y + WHITE_KEY_HEIGHT - BLACK_KEY_HEIGHT;

	  key [note] = new PianoKey(this, key_x, key_y,
				BLACK_KEY_WIDTH,
				BLACK_KEY_HEIGHT,
				BLACK, octave * black_notes [i] );
	}
}
void PianoKeyboard::Display()
{
	// display all the keys
	for (int note = 0; note < KEYS_PER_OCTAVE; note++) {
	  // there's no black between MI and FA
	  if (note == 9) continue;
	  key [note] ->Display();
	}
}

void PianoKeyboard::KeyPressed(int frequency)
{
	EventsHandler->SoundOn(frequency);
}

void PianoKeyboard::KeyReleased(int frequency)
{
	EventsHandler->SoundOff();
}

PianoKeyboard::~PianoKeyboard()
{
	// eliminate all the keys
	for (int note = 0; note < KEYS_PER_OCTAVE; note++) {
	  // there's no black between MI and FA
	  if (note == 9) continue;
	  delete key [note];
	}
}

PianoKey::PianoKey(PianoKeyboard* keyboard,
		   int x, int y, int width, int height,
		   COLOR_TYPE color, int frequency) :
		   Icon(x, y, x + width, y + height)
{
	// save the key attributes
	parent = keyboard;
	key_color = color;
	key_frequency = frequency;

	// create the key
	key = new Box(x1, y1, width, height, color, color);
}

void PianoKey::Display()
{
	key->Display();
}

void PianoKey::LeftButtonPressed()
{
	parent->KeyPressed(key_frequency);
}

void PianoKey::LeftButtonReleased()
{
	parent->KeyReleased(key_frequency);
}

PianoKey::~PianoKey()
{
	delete key;
}
